/*

   menu_applet.c (version 0.3.2)
   Simple application menu applet for ROX desktop environment.

   Copyright (C) 2001 Leandro Pereira <leandro@linuxmag.com.br>

   History:
	   0.1 -> Initial release;
   	   0.2 -> Fixed some small bugs;
                  Added gdk-pixbuf support (image scaling!)
           0.2b-> Fixed makefile;
                  Removed (some) compilation warnings;
                  Added pixmap file verification (through stat());
                  Added a fixed menu position;
                  Implemented first-run configuration file creation
           0.3 -> Implemented groups;
                  Fixed AppRun and Makefile files so it'll compile under
                    other systems than Linux-ix86;
                  Removed more compilation warnings (damn them!)      
	   0.3.1->Added patch from Stephen Watson <watson@ultra.demon.co.uk>
	            (fixes lots of bugs, including the BadMatch error on so-
	            me architetures, removed all compilation warnings (thanks
	            man... I've learned a bit), and other small ones);
	          Implemented "label" (displays a label with a big font on the
	            menu
	   0.3.2->Applied patch from Chis Arndt <chris.arndt@web.de> (refer to
	            Help/Changes for details);
	          Removed get_user_home() and CHOICES_FILE.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h>
#include <malloc.h>
#include <pwd.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "choices.h"

#include "menu.xpm"
#include "menu2.xpm"
#include "null.xpm"
#include "group.xpm"

GtkWidget *menu, *window, *button, *_pixmap, *plug;
GdkFont *label_font;

char *get_app_name(char *path)
{
    char *tmp[2];

    tmp[0] = strtok(path, "/");
    while (tmp[0] != NULL) {
	tmp[0] = strtok(NULL, "/");
	if (tmp[0])
	    tmp[1] = tmp[0];
    }

    return (tmp[1]);
}

/*
 * Thanks to Carlos Pereira for the example (I think I got this
 * at gtk-app-devel list).
 *
 * Someone there could make this function better? I don't know
 * if it'll work with other GTK themes than the one I'm using
 * due to different font sizes.
 *
 * Fixed: borrowed the code from GnomeICU :P
 */
void set_position(GtkMenu * _menu, gint * px, gint * py, gpointer data)
{
    GtkWidget *widget;
    gint sw;
    GtkRequisition requisition;

    widget = GTK_WIDGET(_menu);

    gtk_widget_size_request(widget, &requisition);

    gdk_window_get_origin(button->window, px, py);
    *py -= requisition.height;

    sw = gdk_screen_width();

    if (*px + requisition.width > sw)
	*px -= *px + requisition.width - sw;
}

GtkWidget *create_pixmap(GtkWidget * widget, char *path)
{
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;
    GdkPixbuf *pixbuf, *pixbuf_scaled;
    struct stat info;

    if (!stat(path, &info)) {
	pixbuf = gdk_pixbuf_new_from_file(path);
    } else {
	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) null_xpm);
	fprintf(stderr, "*** Pixmap not found or unable to stat: %s\n", path);
    }
    pixbuf_scaled = gdk_pixbuf_scale_simple(pixbuf, 24, 24,
					    GDK_INTERP_TILES);
    gdk_pixbuf_render_pixmap_and_mask(pixbuf_scaled, &gdkpixmap,
				      &mask, 1);
    pixmap = gtk_pixmap_new(gdkpixmap, mask);

    return pixmap;
}

GtkWidget *create_pixmap_d(GtkWidget * widget, char *from[])
{
    GdkColormap *colormap;
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;

    colormap = gtk_widget_get_colormap(widget);
    gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask,
						      NULL, from);
    pixmap = gtk_pixmap_new(gdkpixmap, mask);
    gdk_pixmap_unref(gdkpixmap);
    gdk_bitmap_unref(mask);

    return pixmap;
}

int create_choices_file(void)
{
    guchar *user_choices;
    FILE *choices;

    user_choices = choices_find_path_save("Apps", "ROX-Menu", TRUE);

    if (user_choices != NULL) {
        fprintf(stderr, "*** %s not found. Creating a default one.\n",
	        user_choices);

        choices = fopen(user_choices, "w+");
        fprintf(choices, "l=Configure me (refer to README for details)\n");
        fprintf(choices, "d=/usr/apps\n");
        fclose(choices);
    }
    else {
        fprintf(stderr, "*** %s not found. Not creating a default one, because you disabled choices saving. Exiting...\n",
	        user_choices);
        gtk_exit(1);
    }
    
    g_free(user_choices);
    return (0);
}

int run_app(GtkWidget * widget, gpointer data)
{
    char tmp[256];
    pid_t child;

    snprintf(tmp, 256, "%s/AppRun", (char*) data);

    child = fork();
    if (child == -1)
	return (0);
    else if (child == 0) {
        char *argv[2];
        argv[0] = tmp;
        argv[1] = 0;
	execv(tmp, argv);
	_exit(127);
    }
    return child;
}

void change_menu_pixmap(char *from[])
{
    gtk_container_remove(GTK_CONTAINER(button), _pixmap);
    _pixmap = create_pixmap_d(button, from);
    gtk_widget_show(_pixmap);
    gtk_container_add(GTK_CONTAINER(button), _pixmap);
}

void pixmaped_menu_item(GtkWidget * menuitem, char *dir, char *from[])
{
    GtkWidget *hbox, *pixmap, *label;
    char icon_path[256];

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_container_add(GTK_CONTAINER(menuitem), hbox);

    snprintf(icon_path, 256, "%s/AppIcon.xpm", dir);
    if (from == NULL) {
	pixmap = create_pixmap(GTK_WIDGET(button), icon_path);
    } else {
	pixmap = create_pixmap_d(GTK_WIDGET(button), from);
    }
    gtk_widget_show(pixmap);
    gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);

    label = gtk_label_new(get_app_name(dir));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

}

int popup_menu(GtkWidget * widget, GdkEvent * event)
{
    GtkWidget *menuitem, *submenu, *submenuitems;
    FILE *apps;
    char buffer[1024], *strip, tmp[256], *val[3],
    buf[256];
    guchar *user_choices;
    struct stat info;
    DIR *dp;
    struct dirent *fp;


    if (GTK_IS_WIDGET(menu))
	gtk_widget_destroy(GTK_WIDGET(menu));

    menu = gtk_menu_new();


    user_choices = choices_find_path_load("Apps", "ROX-Menu");

    if (user_choices == NULL)
	create_choices_file();

    apps = fopen(user_choices, "r");

    while (fgets((char *) buffer, 1024, apps)) {

	if ((strip = strchr((char *) buffer, '\n')))
	    *strip = 0;

	val[0] = strtok(buffer, "=");
	val[1] = strtok(NULL, "=");

	if (!strncmp(buffer, "-", 1)) {	/* Separator */
	    menuitem = gtk_menu_item_new();

	    gtk_widget_show(menuitem);
	    gtk_widget_set_sensitive(GTK_WIDGET(menuitem), FALSE);

	    gtk_menu_append(GTK_MENU(menu), menuitem);
/*
   Choices menu... implement it? :P

   }else if(!strncmp((char*)buffer, "c", 1)){

   menuitem=gtk_menu_item_new();
   pixmaped_menu_item(menuitem, "/Choices", null_xpm);
   gtk_widget_show(menuitem);
   gtk_menu_append(GTK_MENU(menu), menuitem);
 */
	} else if (!strncmp(buffer, "d", 1)) {		/* Group */
	    snprintf(buf, 256, "%s", val[1]);
	    if (!(dp = opendir(val[1])))
		fprintf(stderr, "*** Can't opendir(\"%s\").\n", val[1]);
	    menuitem = gtk_menu_item_new();
	    pixmaped_menu_item(menuitem, val[1], group_xpm);
	    gtk_widget_show(menuitem);
	    gtk_menu_append(GTK_MENU(menu), menuitem);

	    submenu = gtk_menu_new();
	    gtk_widget_show(submenu);
	    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);

	    while (dp && (fp = readdir(dp)) != NULL) {
		snprintf(tmp, 256, "%s/%s/AppRun", buf, fp->d_name);
		if (!stat(tmp, &info)) {
		    submenuitems = gtk_menu_item_new();
		    snprintf(tmp, 256, "%s/%s", buf, fp->d_name);
		    gtk_signal_connect(GTK_OBJECT(submenuitems), "activate",
			(GtkSignalFunc) run_app, g_strdup(tmp));
		    pixmaped_menu_item(submenuitems, tmp, NULL);
		    gtk_widget_show(submenuitems);
		    gtk_menu_append(GTK_MENU(submenu), submenuitems);
		}
	    }

	} else if(!strncmp(buffer, "l", 1)) {
	    GtkStyle *style;
	    GtkWidget *label;
	    
	    label=gtk_label_new(val[1]);
	    gtk_widget_show(GTK_WIDGET(label));
	    
	    menuitem=gtk_menu_item_new();
	    gtk_widget_show(GTK_WIDGET(menuitem));

            style=gtk_style_copy(gtk_widget_get_style(label));
            style->font=label_font;
            gtk_widget_set_style(GTK_WIDGET(label), style);

	    gtk_container_add(GTK_CONTAINER(menuitem), label);

            gtk_menu_append(GTK_MENU(menu), menuitem);
	} else {		/* Normal menu item */
	    menuitem = gtk_menu_item_new();

	    gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
		     (GtkSignalFunc) run_app, g_strdup(val[1]));

	    pixmaped_menu_item(menuitem, val[1], NULL);

	    gtk_widget_show(menuitem);
	    gtk_menu_append(GTK_MENU(menu), menuitem);
	}
    }

    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, set_position,
		   NULL, 1, 0);
    gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
    
    g_free(user_choices);

    return (1);
}


/* This is called as a signal handler */
static void child_died(int signum)
{
	pid_t	child;
	int	status;

	/* Find out which children exited and allow them to die */
	while (1)
	{
		child = waitpid(-1, &status, WNOHANG);

		if (child == 0 || child == -1)
			break;

	}

}

int main(int argc, char *argv[], char * envp[])
{
    struct sigaction	act;

    gtk_init(&argc, &argv);
    gdk_rgb_init();
    gtk_widget_push_visual(gdk_rgb_get_visual());
    gtk_widget_push_colormap(gdk_rgb_get_cmap());
    
    label_font=gdk_font_load("-adobe-helvetica-bold-r-normal-*-*-120-*-*-p-*-iso8859-1");

    if (argc > 1) {
	window = gtk_plug_new(atol(argv[1]));
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			   GTK_SIGNAL_FUNC(gtk_main_quit),
			   "WM destroy");
    } else {
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    }

    /* Let child processes die */
    act.sa_handler = child_died;
    sigemptyset(&act.sa_mask);
    act.sa_flags = SA_NOCLDSTOP;
    sigaction(SIGCHLD, &act, NULL);

    choices_init();

    gtk_widget_set_usize(window, 48, 48);
    gtk_widget_show(window);
    gtk_window_set_title(GTK_WINDOW(window), "ROX Menu");

    button = gtk_button_new();
    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
    GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(button), GTK_CAN_FOCUS);

    _pixmap = create_pixmap_d(window, menu_xpm);
    gtk_widget_show(_pixmap);

    gtk_container_add(GTK_CONTAINER(button), _pixmap);

    gtk_widget_show(button);
    gtk_container_add(GTK_CONTAINER(window), button);

    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			      (GtkSignalFunc) popup_menu, NULL);
    gtk_signal_connect_object(GTK_OBJECT(button), "enter",
		 (GtkSignalFunc) change_menu_pixmap, (gpointer) menu2_xpm);
    gtk_signal_connect_object(GTK_OBJECT(button), "leave",
		  (GtkSignalFunc) change_menu_pixmap, (gpointer) menu_xpm);

    gtk_main();

    return (0);
}
